@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2020 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2024 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -124,17 +123,16 @@
  */
 #define MUNGE_REPLAY_PURGE_SECS         60
 
-/*  Number of attempts to signal a process before sending SIGKILL.
+/*  Maximum number of milliseconds to wait for a process to terminate after
+ *    sending a signal.
  */
-#define MUNGE_SIGNAL_ATTEMPTS           19
+#define MUNGE_SIGNAL_WAIT_MSECS         5000
 
-/*  Starting number of milliseconds between signaling a process and checking
- *    to see if it has responded (i.e., kicked the bucket, shuffled off this
- *    mortal coil, run down the curtain, and joined the bleedin' choir
- *    invisible).  The delay is further incremented by this amount after each
- *    attempt.
+/*  Number of milliseconds between checks to see whether a process has
+ *    terminated (i.e., kicked the bucket, shuffled off this mortal coil,
+ *    run down the curtain, and joined the bleedin' choir invisible).
  */
-#define MUNGE_SIGNAL_DELAY_MSECS        50
+#define MUNGE_SIGNAL_CHECK_MSECS        25
 
 /*  Socket backlog for the server listening on the unix domain socket.
  */
