@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2020 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2024 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -171,8 +170,16 @@ _m_msg_client_connect (m_msg_t m, char *path)
         return (EMUNGE_SOCKET);
     }
     if (stat (path, &st) < 0) {
-        m_msg_set_err (m, EMUNGE_SOCKET,
-            strdupf ("Failed to access \"%s\": %s", path, strerror (errno)));
+        if (errno == ENOENT) {
+            m_msg_set_err (m, EMUNGE_SOCKET,
+                strdupf ("Failed to access \"%s\": %s (%s)",
+                path, strerror (errno), "Did you start munged?"));
+        }
+        else {
+            m_msg_set_err (m, EMUNGE_SOCKET,
+                strdupf ("Failed to access \"%s\": %s",
+                path, strerror (errno)));
+        }
         return (EMUNGE_SOCKET);
     }
     if (!S_ISSOCK (st.st_mode)) {
@@ -199,16 +206,20 @@ _m_msg_client_connect (m_msg_t m, char *path)
     if (addr.sun_path[ sizeof (addr.sun_path) - 1 ] != '\0') {
         close (sd);
         m_msg_set_err (m, EMUNGE_OVERFLOW,
-            strdup ("Exceeded maximum length of socket pathname"));
+            strdupf ("Exceeded maximum length of %lu bytes "
+            "for socket pathname", sizeof (addr.sun_path)));
         return (EMUNGE_OVERFLOW);
     }
     i = 1;
     while (1) {
         /*
-         * If a call to connect() for a Unix domain stream socket finds that
-         *   the listening socket's queue is full, ECONNREFUSED is returned
-         *   immediately.  (cf, Stevens UNPv1, s14.4, p378)
-         * If ECONNREFUSED, try again up to MUNGE_SOCKET_CONNECT_ATTEMPTS.
+         * Retry transient errors up to MUNGE_SOCKET_CONNECT_ATTEMPTS after a
+         *   linearly increasing backoff.
+         * Linux: connect() returns EAGAIN for nonblocking UNIX domain sockets
+         *   if the connection cannot be completed immediately.
+         *   [Linux man-pages 6.05.01]
+         * BSD: connect() returns ECONNREFUSED for UNIX domain stream sockets
+         *   when the listening socket's queue is full. [Stevens UNPv1]
          */
         n = connect (sd, (struct sockaddr *) &addr, sizeof (addr));
 
@@ -218,7 +229,7 @@ _m_msg_client_connect (m_msg_t m, char *path)
         if (errno == EINTR) {
             continue;
         }
-        if (errno != ECONNREFUSED) {
+        if ((errno != EAGAIN) && (errno != ECONNREFUSED)) {
             break;
         }
         if (i >= MUNGE_SOCKET_CONNECT_ATTEMPTS) {
