@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2020 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2024 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -468,7 +467,7 @@ display_help (char *prog)
     printf ("\n");
 
     printf ("  %*s %s\n", w, "-h, --help",
-            "Display this help");
+            "Display this help message");
 
     printf ("  %*s %s\n", w, "-L, --license",
             "Display license information");
@@ -478,21 +477,21 @@ display_help (char *prog)
 
     printf ("\n");
 
-    printf ("  %*s %s\n", w, "-i, --input=FILE",
-            "Input credential from FILE");
+    printf ("  %*s %s\n", w, "-i, --input=PATH",
+            "Input credential from file");
 
     printf ("  %*s %s\n", w, "-n, --no-output",
             "Discard all output");
 
-    printf ("  %*s %s\n", w, "-m, --metadata=FILE",
-            "Output metadata to FILE");
+    printf ("  %*s %s\n", w, "-m, --metadata=PATH",
+            "Output metadata to file");
 
-    printf ("  %*s %s\n", w, "-o, --output=FILE",
-            "Output payload to FILE");
+    printf ("  %*s %s\n", w, "-o, --output=PATH",
+            "Output payload to file");
 
     printf ("\n");
 
-    printf ("  %*s %s\n", w, "-k, --keys=STRING",
+    printf ("  %*s %s\n", w, "-k, --keys=STR",
             "Specify subset of metadata keys to output");
 
     printf ("  %*s %s\n", w, "-K, --list-keys",
@@ -501,8 +500,8 @@ display_help (char *prog)
     printf ("  %*s %s\n", w, "-N, --numeric",
             "Display metadata values numerically");
 
-    printf ("  %*s %s\n", w, "-S, --socket=STRING",
-            "Specify local domain socket for munged");
+    printf ("  %*s %s\n", w, "-S, --socket=PATH",
+            "Specify local socket for munged");
 
     printf ("\n");
     printf ("By default, credential read from stdin, "
