@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2020 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2024 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -44,6 +43,7 @@
 #include <time.h>
 #include <unistd.h>
 #include <munge.h>
+#include "clock.h"
 #include "conf.h"
 #include "license.h"
 #include "lock.h"
@@ -77,7 +77,8 @@
 #define OPT_SEED_FILE           268
 #define OPT_TRUSTED_GROUP       269
 #define OPT_ORIGIN              270
-#define OPT_LAST                271
+#define OPT_LISTEN_BACKLOG      271
+#define OPT_LAST                272
 
 const char * const short_opts = ":hLVfFMsS:v";
 
@@ -101,6 +102,7 @@ struct option long_opts[] = {
     { "group-check-mtime", required_argument, NULL, OPT_GROUP_CHECK   },
     { "group-update-time", required_argument, NULL, OPT_GROUP_UPDATE  },
     { "key-file",          required_argument, NULL, OPT_KEY_FILE      },
+    { "listen-backlog",    required_argument, NULL, OPT_LISTEN_BACKLOG},
     { "log-file",          required_argument, NULL, OPT_LOG_FILE      },
     { "max-ttl",           required_argument, NULL, OPT_MAX_TTL       },
     { "num-threads",       required_argument, NULL, OPT_NUM_THREADS   },
@@ -117,12 +119,19 @@ struct option long_opts[] = {
  *  Internal Prototypes
  *****************************************************************************/
 
+static void _conf_set_cwd (conf_t conf);
+
 static void _conf_display_help (char *prog);
 
+static void _conf_set_string (char **str_p, const char *val, const char *cwd,
+        const char *dsc);
+
 static void _conf_process_stop (conf_t conf);
 
 static int _conf_send_signal (pid_t pid, int signum, int msecs);
 
+static void _conf_sleep (int msecs);
+
 static void _conf_set_origin_addr (conf_t conf);
 
 static int _conf_open_keyfile (const char *keyfile, int got_force);
@@ -170,26 +179,29 @@ create_conf (void)
     conf->config_name = NULL;
     conf->lockfile_fd = -1;
     conf->lockfile_name = NULL;
+    conf->listen_backlog = MUNGE_SOCKET_BACKLOG;
+
+    _conf_set_cwd (conf);
 
     if (!(conf->logfile_name = strdup (MUNGE_LOGFILE_PATH))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Failed to copy logfile name string");
+            "Failed to copy log-file name default string");
     }
     if (!(conf->pidfile_name = strdup (MUNGE_PIDFILE_PATH))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Failed to copy pidfile name string");
+            "Failed to copy pid-file name default string");
     }
     if (!(conf->socket_name = strdup (MUNGE_SOCKET_NAME))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Failed to copy socket name string");
+            "Failed to copy socket name default string");
     }
     if (!(conf->seed_name = strdup (MUNGE_SEEDFILE_PATH))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Failed to copy seed name string");
+            "Failed to copy seed-file name default string");
     }
     if (!(conf->key_name = strdup (MUNGE_KEYFILE_PATH))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-            "Failed to copy key name string");
+            "Failed to copy key-file name default string");
     }
     conf->dek_key = NULL;
     conf->dek_key_len = 0;
@@ -223,10 +235,16 @@ create_conf (void)
 void
 destroy_conf (conf_t conf, int do_unlink)
 {
+    int rv;
+
     assert (conf != NULL);
     assert (conf->ld < 0);              /* sock_destroy() already called */
     assert (conf->lockfile_fd < 0);
 
+    if (conf->cwd) {
+        free (conf->cwd);
+        conf->cwd = NULL;
+    }
     if (conf->config_name) {
         free (conf->config_name);
         conf->config_name = NULL;
@@ -241,7 +259,14 @@ destroy_conf (conf_t conf, int do_unlink)
     }
     if (conf->pidfile_name) {
         if (do_unlink) {
-            (void) unlink (conf->pidfile_name);
+            do {
+                rv = unlink (conf->pidfile_name);
+            } while ((rv < 0) && (errno == EINTR));
+
+            if (rv < 0) {
+                log_msg (LOG_WARNING, "Failed to remove pidfile \"%s\": %s",
+                        conf->pidfile_name, strerror (errno));
+            }
         }
         free (conf->pidfile_name);
         conf->pidfile_name = NULL;
@@ -337,11 +362,8 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 conf->got_stop = 1;
                 break;
             case 'S':
-                if (conf->socket_name)
-                    free (conf->socket_name);
-                if (!(conf->socket_name = strdup (optarg)))
-                    log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Failed to copy socket name string");
+                _conf_set_string (&conf->socket_name, optarg, conf->cwd,
+                        "socket name");
                 break;
             case 'v':
                 conf->got_verbose = 1;
@@ -351,18 +373,12 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 exit (42);
 #if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
             case OPT_AUTH_SERVER:
-                if (conf->auth_server_dir)
-                    free (conf->auth_server_dir);
-                if (!(conf->auth_server_dir = strdup (optarg)))
-                    log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Failed to copy auth-server-dir cmdline string");
+                _conf_set_string (&conf->auth_server_dir, optarg, conf->cwd,
+                        "auth-server-dir name");
                 break;
             case OPT_AUTH_CLIENT:
-                if (conf->auth_client_dir)
-                    free (conf->auth_client_dir);
-                if (!(conf->auth_client_dir = strdup (optarg)))
-                    log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Failed to copy auth-client-dir cmdline string");
+                _conf_set_string (&conf->auth_client_dir, optarg, conf->cwd,
+                        "auth-client-dir name");
                 break;
 #endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
             case OPT_BENCHMARK:
@@ -390,18 +406,31 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 conf->gids_update_secs = l;
                 break;
             case OPT_KEY_FILE:
-                if (conf->key_name)
-                    free (conf->key_name);
-                if (!(conf->key_name = strdup (optarg)))
-                    log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Failed to copy key-file name string");
+                _conf_set_string (&conf->key_name, optarg, conf->cwd,
+                        "key-file name");
+                break;
+            case OPT_LISTEN_BACKLOG:
+                errno = 0;
+                l = strtol (optarg, &p, 10);
+                if (((errno == ERANGE) && ((l == LONG_MIN) || (l == LONG_MAX)))
+                        || (optarg == p) || (*p != '\0')
+                        || (l < -1) || (l > INT_MAX)) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Invalid value \"%s\" for listen-backlog", optarg);
+                }
+                else if (l == 0) {
+                    conf->listen_backlog = MUNGE_SOCKET_BACKLOG;
+                }
+                else if (l == -1) {
+                    conf->listen_backlog = SOMAXCONN;
+                }
+                else {
+                    conf->listen_backlog = l;
+                }
                 break;
             case OPT_LOG_FILE:
-                if (conf->logfile_name)
-                    free (conf->logfile_name);
-                if (!(conf->logfile_name = strdup (optarg)))
-                    log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Failed to copy log-file name string");
+                _conf_set_string (&conf->logfile_name, optarg, conf->cwd,
+                        "log-file name");
                 break;
             case OPT_MAX_TTL:
                 l = strtol (optarg, &p, 10);
@@ -425,25 +454,15 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 conf->nthreads = l;
                 break;
             case OPT_ORIGIN:
-                if (conf->origin_name)
-                    free (conf->origin_name);
-                if (!(conf->origin_name = strdup (optarg)))
-                    log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Failed to copy origin string");
+                _conf_set_string (&conf->origin_name, optarg, NULL, "origin");
                 break;
             case OPT_PID_FILE:
-                if (conf->pidfile_name)
-                    free (conf->pidfile_name);
-                if (!(conf->pidfile_name = strdup (optarg)))
-                    log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Failed to copy pid-file name string");
+                _conf_set_string (&conf->pidfile_name, optarg, conf->cwd,
+                        "pid-file name");
                 break;
             case OPT_SEED_FILE:
-                if (conf->seed_name)
-                    free (conf->seed_name);
-                if (!(conf->seed_name = strdup (optarg)))
-                    log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
-                        "Failed to copy seed-file name string");
+                _conf_set_string (&conf->seed_name, optarg, conf->cwd,
+                        "seed-file name");
                 break;
             case OPT_SYSLOG:
                 conf->got_syslog = 1;
@@ -641,12 +660,59 @@ create_subkeys (conf_t conf)
  *  Internal Functions
  *****************************************************************************/
 
+static void
+_conf_set_cwd (conf_t conf)
+{
+/*  Set the current working directory in order to fix relative paths so these
+ *    locations can still be accessed after chdir() is called.
+ */
+    char  buf [PATH_MAX];
+    char *rv;
+
+    rv = getcwd (buf, sizeof (buf));
+
+    /*  Starting in Linux 2.6.36, the path returned by getcwd() will be
+     *    prefixed with "(unreachable)" if the current directory is not below
+     *    the root directory of the current process.  With glibc 2.27 onwards,
+     *    this will instead result in failure with ENOENT.
+     *  Since relative paths break the getcwd() API contract requiring an
+     *    absolute path, convert anything not starting with an absolute path
+     *    (like "(unreachable)") to ENOENT.
+     */
+    if ((rv != NULL) && (buf[0] != '/')) {
+        rv = NULL;
+        errno = ENOENT;
+    }
+
+    if (rv == NULL) {
+        conf->cwd = NULL;
+        if (errno == ERANGE) {
+            log_msg (LOG_WARNING,
+                    "Failed to set current working directory: "
+                    "Exceeded %lu-byte buffer", sizeof (buf));
+        }
+        else {
+            log_msg (LOG_WARNING,
+                    "Failed to set current working directory: %s",
+                    strerror (errno));
+        }
+    }
+    else {
+        conf->cwd = strdup (buf);
+        if (conf->cwd == NULL) {
+            log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
+                    "Failed to copy current working directory string");
+        }
+    }
+}
+
+
 static void
 _conf_display_help (char *prog)
 {
 /*  Displays a help message describing the command-line options.
  */
-    const int w = -24;                  /* pad for width of option string */
+    const int w = -25;                  /* pad for width of option string */
 
     assert (prog != NULL);
 
@@ -654,7 +720,7 @@ _conf_display_help (char *prog)
     printf ("\n");
 
     printf ("  %*s %s\n", w, "-h, --help",
-            "Display this help");
+            "Display this help message");
 
     printf ("  %*s %s\n", w, "-L, --license",
             "Display license information");
@@ -699,23 +765,26 @@ _conf_display_help (char *prog)
             w, "--group-check-mtime=BOOL", GIDS_GROUP_FILE,
             MUNGE_GROUP_STAT_FLAG);
 
-    printf ("  %*s %s [%d]\n", w, "--group-update-time=INT",
+    printf ("  %*s %s [%d]\n", w, "--group-update-time=SECS",
             "Specify seconds between group info updates",
             MUNGE_GROUP_UPDATE_SECS);
 
     printf ("  %*s %s [%s]\n", w, "--key-file=PATH",
             "Specify key file", MUNGE_KEYFILE_PATH);
 
+    printf ("  %*s %s [%d]\n", w, "--listen-backlog=INT",
+            "Specify listen backlog limit of socket", MUNGE_SOCKET_BACKLOG);
+
     printf ("  %*s %s [%s]\n", w, "--log-file=PATH",
             "Specify log file", MUNGE_LOGFILE_PATH);
 
-    printf ("  %*s %s [%d]\n", w, "--max-ttl=INT",
+    printf ("  %*s %s [%d]\n", w, "--max-ttl=SECS",
             "Specify maximum time-to-live (in seconds)", MUNGE_MAXIMUM_TTL);
 
     printf ("  %*s %s [%d]\n", w, "--num-threads=INT",
             "Specify number of threads to spawn", MUNGE_THREADS);
 
-    printf ("  %*s %s\n", w, "--origin=ADDRESS",
+    printf ("  %*s %s\n", w, "--origin=ADDR",
             "Specify origin address via hostname/IPaddr/interface");
 
     printf ("  %*s %s [%s]\n", w, "--pid-file=PATH",
@@ -727,7 +796,7 @@ _conf_display_help (char *prog)
     printf ("  %*s %s\n", w, "--syslog",
             "Redirect log messages to syslog");
 
-    printf ("  %*s %s\n", w, "--trusted-group=GROUP",
+    printf ("  %*s %s\n", w, "--trusted-group=GID",
             "Specify trusted group/GID for directory checks");
 
     printf ("\n");
@@ -735,65 +804,86 @@ _conf_display_help (char *prog)
 }
 
 
+static void
+_conf_set_string (char **str_p, const char *val, const char *cwd,
+        const char *dsc)
+{
+/*  Set the string referenced by [str_p] to the string value [val], freeing
+ *    an existing string pointed to by [str_p] if needed.  If [val] is not an
+ *    absolute pathname and the current working directory string [cwd] is set,
+ *    that string will be prepended to [val].
+ *  The description string [dsc] will be used in the error message if needed.
+ */
+    assert (str_p != NULL);
+    assert (val != NULL);
+    assert (dsc != NULL);
+
+    if (*str_p != NULL) {
+        free (*str_p);
+    }
+    *str_p = (val[0] == '/' || cwd == NULL)
+        ? strdup (val)
+        : strdupf ("%s/%s", cwd, val);
+
+    if (*str_p == NULL) {
+        log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Failed to copy %s string", dsc);
+    }
+}
+
+
 static void
 _conf_process_stop (conf_t conf)
 {
-/*  Processes the -s/--stop option.
- *  A series of SIGTERMs are sent to the process holding the write-lock.
- *    If the process fails to terminate, a final SIGKILL is sent.
+/*  Process the -s/--stop option.
+ *  A SIGTERM is sent to the process holding the write-lock.
+ *    A SIGKILL is sent afterwards if the process fails to terminate.
  */
     pid_t pid;
-    int   signum;
-    int   msecs;
-    int   i;
     int   rv;
 
     assert (conf != NULL);
-    assert (MUNGE_SIGNAL_ATTEMPTS > 0);
-    assert (MUNGE_SIGNAL_DELAY_MSECS > 0);
+    assert (MUNGE_SIGNAL_WAIT_MSECS > 0);
 
+    /*  Obtain pid of daemon bound to socket.
+     */
     pid = lock_query (conf);
     if (pid <= 0) {
         if (conf->got_verbose) {
             log_err (EMUNGE_SNAFU, LOG_ERR,
-                    "Failed to query socket lockfile \"%s\": %s",
+                    "Failed to query socket lockfile \"%s\": %s (%s)",
                     conf->lockfile_name,
-                    (pid == 0) ? "Lock not held" : strerror (errno));
+                    (pid == 0) ? "Lock not held" : strerror (errno),
+                    "Cannot find running process");
         }
         exit (EXIT_FAILURE);
     }
-    rv = kill (pid, 0);
-    if (rv < 0) {
+    /*  Terminate.
+     */
+    rv = _conf_send_signal (pid, SIGTERM, MUNGE_SIGNAL_WAIT_MSECS);
+    if (rv == 0) {
         if (conf->got_verbose) {
-            log_errno (EMUNGE_SNAFU, LOG_ERR,
-                    "Failed to signal daemon bound to socket \"%s\" (pid %d)",
+            log_msg (LOG_NOTICE,
+                    "Terminated daemon bound to socket \"%s\" (pid %d)",
                     conf->socket_name, pid);
         }
-        exit (EXIT_FAILURE);
+        exit (EXIT_SUCCESS);
     }
-    signum = SIGTERM;
-    msecs = 0;
-    for (i = 0; i < (MUNGE_SIGNAL_ATTEMPTS + 1); i++) {
-        if (i == MUNGE_SIGNAL_ATTEMPTS) {
-            signum = SIGKILL;           /* Kill me harder! */
-        }
-        msecs += MUNGE_SIGNAL_DELAY_MSECS;
-        rv = _conf_send_signal (pid, signum, msecs);
-        if (rv == 0) {
-            if (conf->got_verbose) {
-                log_msg (LOG_NOTICE,
-                        "%s daemon bound to socket \"%s\" (pid %d)",
-                        (signum == SIGTERM) ? "Terminated" : "Killed",
-                        conf->socket_name, pid);
-            }
-            exit (EXIT_SUCCESS);
+    /*  Kill.
+     */
+    rv = _conf_send_signal (pid, SIGKILL, MUNGE_SIGNAL_WAIT_MSECS);
+    if (rv == 0) {
+        if (conf->got_verbose) {
+            log_msg (LOG_NOTICE,
+                    "Killed daemon bound to socket \"%s\" (pid %d)",
+                    conf->socket_name, pid);
         }
+        exit (EXIT_SUCCESS);
     }
-    if (conf->got_verbose) {
-        log_err (EMUNGE_SNAFU, LOG_ERR,
-                "Failed to terminate daemon bound to socket \"%s\" (pid %d)",
-                conf->socket_name, pid);
-    }
+    /*  Fail.
+     */
+    log_err (EMUNGE_SNAFU, LOG_ERR,
+            "Failed to terminate daemon bound to socket \"%s\" (pid %d)",
+            conf->socket_name, pid);
     exit (EXIT_FAILURE);
 }
 
@@ -801,51 +891,94 @@ _conf_process_stop (conf_t conf)
 static int
 _conf_send_signal (pid_t pid, int signum, int msecs)
 {
-/*  Sends the signal [signum] to the process specified by [pid].
- *  Returns 1 if the process is still running after a delay of [msecs],
- *    or 0 if the process cannot be found.
+/*  Send the signal [signum] to the process specified by [pid].
+ *  Return 1 if the process is still running after a delay of [msecs],
+ *    or 0 if the process has terminated.
  */
-    struct timespec ts;
+    struct timespec wait_abstime;
     int             rv;
+    int             sig;
 
     assert (pid > 0);
     assert (signum > 0);
     assert (msecs > 0);
+    assert (MUNGE_SIGNAL_CHECK_MSECS > 0);
 
-    log_msg (LOG_DEBUG, "Signaling pid %d with sig %d and %dms delay",
-            pid, signum, msecs);
-
-    rv = kill (pid, signum);
+    rv = clock_get_timespec (&wait_abstime, msecs);
     if (rv < 0) {
-        if (errno == ESRCH) {
-            return (0);
-        }
         log_errno (EMUNGE_SNAFU, LOG_ERR,
-                "Failed to signal daemon (pid %d, sig %d)", pid, signum);
+                "Failed to get wait time for termination");
     }
-    ts.tv_sec = msecs / 1000;
-    ts.tv_nsec = (msecs % 1000) * 1000 * 1000;
-retry:
-    rv = nanosleep (&ts, &ts);
-    if (rv < 0) {
-        if (errno == EINTR) {
-            goto retry;
+    sig = signum;
+    while (1) {
+        rv = kill (pid, sig);
+        if (rv < 0) {
+            if (errno == ESRCH) {
+                return (0);
+            }
+            log_errno (EMUNGE_SNAFU, LOG_ERR,
+                    "Failed to signal daemon (pid %d, sig %d)", pid, sig);
         }
-        log_errno (EMUNGE_SNAFU, LOG_ERR,
-                "Failed to sleep while awaiting signal result");
-    }
-    rv = kill (pid, 0);
-    if (rv < 0) {
-        if (errno == ESRCH) {
-            return (0);
+        rv = clock_is_timespec_expired (&wait_abstime);
+        if (rv < 0) {
+            log_errno (EMUNGE_SNAFU, LOG_ERR,
+                    "Failed to check if termination wait time has expired");
         }
-        log_errno (EMUNGE_SNAFU, LOG_ERR,
-                "Failed to check daemon (pid %d, sig 0)", pid);
+        if (rv == 1) {
+            break;
+        }
+        /*  Now that SIGNUM has been sent, switch to sending the null signal
+         *    to check for PID existence.
+         */
+        if (sig != 0) {
+            sig = 0;
+        }
+        _conf_sleep (MUNGE_SIGNAL_CHECK_MSECS);
     }
     return (1);
 }
 
 
+static void
+_conf_sleep (int msecs)
+{
+/*  Sleep for the specified number of milliseconds [msecs].
+ */
+#if HAVE_CLOCK_NANOSLEEP
+    struct timespec check_abstime;
+    int             rv;
+
+    rv = clock_get_timespec (&check_abstime, msecs);
+    if (rv < 0) {
+        log_errno (EMUNGE_SNAFU, LOG_ERR,
+                "Failed to get check time for termination");
+    }
+    do {
+        rv = clock_nanosleep (CLOCK_REALTIME, TIMER_ABSTIME, &check_abstime,
+                NULL);
+    } while ((rv < 0) && (errno == EINTR));
+    if (rv < 0) {
+        log_errno (EMUNGE_SNAFU, LOG_ERR,
+                "Failed to sleep before checking for termination");
+    }
+#else  /* !HAVE_CLOCK_NANOSLEEP */
+    struct timespec check_reltime;
+    int             rv;
+
+    check_reltime.tv_sec = msecs / 1000;
+    check_reltime.tv_nsec = (msecs % 1000) * 1000 * 1000;
+
+    do {
+        rv = nanosleep (&check_reltime, &check_reltime);
+    } while ((rv < 0) && (errno == EINTR));
+    if (rv < 0) {
+        log_errno (EMUNGE_SNAFU, LOG_ERR,
+                "Failed to sleep before checking for termination");
+    }
+#endif /* !HAVE_CLOCK_NANOSLEEP */
+}
+
+
 static void
 _conf_set_origin_addr (conf_t conf)
 {
@@ -907,8 +1040,9 @@ _conf_open_keyfile (const char *keyfile, int got_force)
     is_symlink = (lstat (keyfile, &st) == 0) ? S_ISLNK (st.st_mode) : 0;
 
     if (stat (keyfile, &st) < 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR,
-            "Failed to check keyfile \"%s\"", keyfile);
+        log_err (EMUNGE_SNAFU, LOG_ERR,
+            "Failed to find keyfile \"%s\": %s (%s)",
+            keyfile, strerror (errno), "Did you run mungekey?");
     }
     if (!S_ISREG (st.st_mode)) {
         log_err (EMUNGE_SNAFU, LOG_ERR,
