@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2020 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2024 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -58,12 +57,14 @@ struct conf {
     munge_mac_t     def_mac;            /* default message auth code type    */
     munge_ttl_t     def_ttl;            /* default time-to-live in seconds   */
     munge_ttl_t     max_ttl;            /* maximum time-to-live in seconds   */
+    char           *cwd;                /* current working dir at startup    */
     char           *config_name;        /* configuration filename            */
     int             lockfile_fd;        /* daemon lockfile fd                */
     char           *lockfile_name;      /* daemon lockfile name              */
     char           *logfile_name;       /* daemon logfile name               */
     char           *pidfile_name;       /* daemon pidfile name               */
     char           *socket_name;        /* unix domain socket filename       */
+    int             listen_backlog;     /* unix domain socket listen backlog */
     char           *seed_name;          /* random seed filename              */
     char           *key_name;           /* symmetric key filename            */
     unsigned char  *dek_key;            /* subkey for cipher ops             */
