@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2020 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2024 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -264,6 +263,7 @@ _random_read_entropy_from_file (const char *path)
     int  n;
     char dir [PATH_MAX];
     char ebuf [1024];
+    int  rv;
 
     if ((path == NULL) || (path[0] == '\0')) {
         errno = EINVAL;
@@ -290,13 +290,16 @@ _random_read_entropy_from_file (const char *path)
 
     n = _random_read_seed (path, RANDOM_SEED_BYTES);
     if (n < 0) {
-        if (unlink (path) < 0) {
-            if (errno != ENOENT) {
-                log_msg (LOG_WARNING,
-                        "Failed to remove insecure PRNG seed \"%s\"", path);
-            }
+        do {
+            rv = unlink (path);
+        } while ((rv < 0) && (errno == EINTR));
+
+        if ((rv < 0) && (errno != ENOENT)) {
+            log_msg (LOG_WARNING,
+                    "Failed to remove insecure PRNG seed \"%s\": %s",
+                    path, strerror (errno));
         }
-        else {
+        else if (rv == 0) {
             log_msg (LOG_INFO, "Removed insecure PRNG seed \"%s\"", path);
         }
         n = 0;
@@ -352,13 +355,11 @@ _random_read_seed (const char *path, int num_bytes)
                 path);
         return (-1);
     }
+    do {
+        fd = open (path, O_RDONLY);
+    } while ((fd < 0) && (errno == EINTR));
 
-retry_open:
-    fd = open (path, O_RDONLY);
     if (fd < 0) {
-        if (errno == EINTR) {
-            goto retry_open;
-        }
         if (errno == ENOENT) {
             return (0);
         }
@@ -429,6 +430,7 @@ _random_write_seed (const char *path, int num_bytes)
 /*  Writes 'num_bytes' of random bytes to the seed file specified by 'path'.
  *  Returns the number of bytes written, or -1 on error.
  */
+    int            rv;
     int            fd;
     int            num_left;
     int            num_want;
@@ -438,16 +440,19 @@ _random_write_seed (const char *path, int num_bytes)
     assert (path != NULL);
     assert (num_bytes > 0);
 
-    if ((unlink (path) < 0) && (errno != ENOENT)) {
+    do {
+        rv = unlink (path);
+    } while ((rv < 0) && (errno == EINTR));
+
+    if ((rv < 0) && (errno != ENOENT)) {
         log_msg (LOG_WARNING, "Failed to unlink old PRNG seed \"%s\": %s",
                 path, strerror (errno));
     }
-retry_open:
-    fd = open (path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+    do {
+        fd = open (path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+    } while ((fd < 0) && (errno == EINTR));
+
     if (fd < 0) {
-        if (errno == EINTR) {
-            goto retry_open;
-        }
         log_msg (LOG_WARNING, "Failed to create PRNG seed \"%s\": %s",
                 path, strerror (errno));
         return (-1);
